package com.hai.service.impl;

import com.alibaba.fastjson.JSON;
import com.hai.common.Base64Util;
import com.hai.common.QRCodeGenerator;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.config.CommonSysConst;
import com.hai.config.WeChatQrcodeUtils;
import com.hai.dao.HighDiscountAgentRelMapper;
import com.hai.entity.*;
import com.hai.enum_type.WxQrCodeTypeEnum;
import com.hai.service.*;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.util.*;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/4/4 00:15
 */
@Service("highDiscountAgentRelService")
public class HighDiscountAgentRelServiceImpl implements HighDiscountAgentRelService {

    @Resource
    private HighDiscountAgentRelMapper highDiscountAgentRelMapper;

    @Resource
    private HighAgentService highAgentService;

    @Resource
    private HighDiscountService highDiscountService;

    @Resource
    private HighDiscountAgentCodeService highDiscountAgentCodeService;

    @Resource
    private HighDiscountAgentBatchService discountAgentBatchService;

    @Autowired
    private WeChatQrcodeUtils weChatQrcodeUtils;

    @Override
    @Transactional(propagation= Propagation.REQUIRED)
    public void insertDiscountAgentRel(HighDiscountAgentRel highDiscountAgentRel,Integer stockCount) throws Exception {
        if (highDiscountAgentRel.getId() != null) {
            highDiscountAgentRelMapper.updateByPrimaryKey(highDiscountAgentRel);
        } else {
            highDiscountAgentRelMapper.insert(highDiscountAgentRel);
        }
        HighDiscountAgentBatch discountAgentBatch = new HighDiscountAgentBatch();
        discountAgentBatch.setDiscountAgentId(highDiscountAgentRel.getId());
        discountAgentBatch.setDiscountId(highDiscountAgentRel.getDiscountId());
        discountAgentBatch.setDiscountName(highDiscountAgentRel.getHighDiscount().getDiscountName());
        discountAgentBatch.setDiscountPrice(highDiscountAgentRel.getHighDiscount().getDiscountPrice());
        discountAgentBatch.setUseScope(highDiscountAgentRel.getHighDiscount().getUseScope());
        discountAgentBatch.setUsingRange(highDiscountAgentRel.getHighDiscount().getUsingRange());
        discountAgentBatch.setAgentId(highDiscountAgentRel.getAgentId());
        discountAgentBatch.setBatchNum(stockCount);
        discountAgentBatchService.editData(discountAgentBatch);

        discountAgentBatch.setBatchIdentifier((10000 + discountAgentBatch.getId())+"");
        discountAgentBatchService.editData(discountAgentBatch);

        HighDiscountAgentCode code;
        Map<String,Object> map = new HashMap<>();
        map.put("type" , "DISCOUNT");
        for (int i = 0;i < stockCount;i++) {
            code = new HighDiscountAgentCode();
            code.setDiscountAgentId(highDiscountAgentRel.getId());
            code.setDiscountAgentBatchId(discountAgentBatch.getId());
            code.setStatus(1);
            code.setCreateTime(new Date());
            highDiscountAgentCodeService.insertCode(code);
            // 二维码参数
            map.put("id", code.getId());

            // 生成二维码
            String qrCodeUrl = CommonSysConst.getSysConfig().getHsgDomainName() + "/hsgH5?accountId=0000011&key=&code=" + Base64Util.encode(AESEncodeUtil.aesEncrypt(JSON.toJSONString(map)));
            code.setExt1(qrCodeUrl);
            // 生成二维码地址
            highDiscountAgentCodeService.updateCode(code);

            if (i == 0) {
                discountAgentBatch.setCodeStart(code.getId().toString());
            } else if ((i + 1) == stockCount) {
                discountAgentBatch.setCodeEnd(code.getId().toString());
            }
        }
        discountAgentBatchService.editData(discountAgentBatch);
    }

    @Override
    public void updateDiscountAgentRel(HighDiscountAgentRel highDiscountAgentRel) {
        highDiscountAgentRelMapper.updateByPrimaryKey(highDiscountAgentRel);
    }

    @Override
    public void deleteDiscountAgentRel(Long id) {
        highDiscountAgentRelMapper.deleteByPrimaryKey(id);
    }

    @Override
    public HighDiscountAgentRel getRelByDiscountAgent(Long discountId, Long agentId) {
        HighDiscountAgentRelExample example = new HighDiscountAgentRelExample();
        example.createCriteria().andDiscountIdEqualTo(discountId).andAgentIdEqualTo(agentId);
        List<HighDiscountAgentRel> list = highDiscountAgentRelMapper.selectByExample(example);
        if (list != null && list.size() >0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public List<HighDiscountAgentRel> getRelByDiscountId(Long discountId) {
        HighDiscountAgentRelExample example = new HighDiscountAgentRelExample();
        example.createCriteria().andDiscountIdEqualTo(discountId);
        List<HighDiscountAgentRel> list = highDiscountAgentRelMapper.selectByExample(example);
        return list;
    }

    @Override
    public HighDiscountAgentRel getRelById(Long id) {
        HighDiscountAgentRel rel = highDiscountAgentRelMapper.selectByPrimaryKey(id);
        if (rel != null) {
            rel.setHighDiscount(highDiscountService.getDiscountById(rel.getDiscountId()));
            rel.setHighAgent(highAgentService.findByAgentMsgId(rel.getAgentId()));
        }
        return rel;
    }

    @Override
    public List<HighDiscountAgentRel> getRelList(Map<String, Object> map) {
        HighDiscountAgentRelExample example = new HighDiscountAgentRelExample();
        HighDiscountAgentRelExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "discountId") != null) {
            criteria.andDiscountIdEqualTo(MapUtils.getLong(map, "discountId"));
        }

        if (MapUtils.getLong(map, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(map, "agentId"));
        }

        criteria.andStatusEqualTo(1);
        example.setOrderByClause("create_time desc");
        List<HighDiscountAgentRel> list = highDiscountAgentRelMapper.selectByExample(example);
        for (HighDiscountAgentRel rel : list) {
            rel.setHighDiscount(highDiscountService.getDiscountById(rel.getDiscountId()));
            rel.setHighAgent(highAgentService.findByAgentMsgId(rel.getAgentId()));
            // rel.setTotalCount(highDiscountAgentCodeService.getTotalDiscount(rel.getId()));
            // rel.setSalesCount(highDiscountAgentCodeService.getSalesDiscount(rel.getId()));
        }
        return list;
    }

    @Override
    public List<HighDiscountAgentRel> getDiscountUseCount(Map<String, Object> map) {
        HighDiscountAgentRelExample example = new HighDiscountAgentRelExample();
        HighDiscountAgentRelExample.Criteria criteria = example.createCriteria().andStatusEqualTo(1);


        example.setOrderByClause("create_time desc");
        return highDiscountAgentRelMapper.selectByExample(example);
    }
}
