package com.v1.controller.business;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.ApiMchProduct;
import com.hai.entity.ApiMemberProduct;
import com.hai.entity.ApiMerchants;
import com.hai.model.ResponseData;
import com.hai.openApi.config.PriceComputeConfig;
import com.hai.openApi.service.ApiMchProductService;
import com.hai.order.type.OrderProductType;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "/memberRecharge")
@Api(value = "会员充值业务")
public class MemberProductsController {

    private static Logger log = LoggerFactory.getLogger(MemberProductsController.class);

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private ApiIpAddressService apiIpAddressService;

    @Resource
    private ApiProductService apiProductService;

    @Resource
    private PriceComputeConfig priceComputeConfig;

    @Resource
    private ApiMchProductService apiMchProductService;

    @Resource
    private ApiMemberProductService apiMemberProductService;


    @RequestMapping(value = "/memberProducts", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询会员产品")
    public ResponseData memberProducts(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!apiIpAddressService.validationIpAddressLegal(object.getString("merchId") , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.IP_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (
                    StringUtils.isBlank(object.getString("merchId")) ||
                    StringUtils.isBlank(object.getString("sign"))
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "帐户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_NOT_EXIST);
            }

            if (!secConfigService.isConfig(OrderProductType.PRODUCT_TYPE3.getApiCode(), object.getString("merchId"))) {
                log.error("getAllCities error!", "当前帐户无权限查询！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOT_AUTHORIZED, "");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , apiMerchants.getApiKey());
            map.put("merchId" , object.getString("merchId"));

            if (!secConfigService.isSignVerify(map , object.getString("sign"))) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }


            List<ApiMemberProduct> list = apiMemberProductService.getListApiMemberProduct(new HashMap<>());

            Map<String, Object> mapProduct = new HashMap<>();
            mapProduct.put("mchId" , object.getString("merchId"));
            mapProduct.put("productType" , OrderProductType.PRODUCT_TYPE3.getNumber());

            ApiMchProduct apiMchProducts = apiMchProductService.findByMap(mapProduct);


            for (ApiMemberProduct apiMemberProduct : list) {
                apiMemberProduct.setCostPrice(priceComputeConfig.computationPriceCost(apiMchProducts.getDiscount() , apiMemberProduct.getPrice() , apiMemberProduct.getCostPrice()));
            }

            return ResponseMsgUtil.success(list);

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
